package com.logicbig.example;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.Query;
import java.util.Arrays;
import java.util.List;

public class ExampleMain {

    public static void main(String[] args) {
        EntityManagerFactory emf = Persistence.createEntityManagerFactory("test1");
        try {
            printTables(emf);
            persistCustomerEntity(emf);
            persistOrderItemEntity(emf);
            nativeQueries(emf);
            loadEntities(emf);
        } finally {
            emf.close();
        }
    }

    private static void printTables(EntityManagerFactory emf) {
        EntityManager em = emf.createEntityManager();
        nativeQuery(em, "SHOW TABLES");
        nativeQuery(em, "SHOW COLUMNS from Customer");
        nativeQuery(em, "SHOW COLUMNS from OrderItem");
    }

    private static void nativeQueries(EntityManagerFactory emf) {
        System.out.println(" -- native queries --");
        EntityManager em = emf.createEntityManager();
        nativeQuery(em, "Select * from Customer");
        nativeQuery(em, "Select * from OrderItem");
        em.close();
    }

    private static void persistCustomerEntity(EntityManagerFactory emf) {
        System.out.println("-- Persisting Customer entity --");
        EntityManager em = emf.createEntityManager();

        Customer customer = new Customer();
        customer.setName("Maria Dorsey");
        customer.setAddress("1719 Shumaker Boulevard, Sylvania");
        customer.addOrderItem("GEN SSD", 2);
        customer.addOrderItem("SK Monitor", 3);

        em.getTransaction().begin();
        em.persist(customer);
        em.getTransaction().commit();

        em.close();
    }

    private static void persistOrderItemEntity(EntityManagerFactory emf) {
        System.out.println("-- Persisting OrderItem entity --");
        EntityManager em = emf.createEntityManager();

        OrderItem orderItem = new OrderItem();
        orderItem.setItemName("QK USB Drive");
        orderItem.setQuantity(5);

        Customer customer = new Customer();
        customer.setName("Heather Barton");
        customer.setAddress("563 Holbein Square, Woodbine");
        customer.addOrderItem(orderItem);

        em.getTransaction().begin();
        em.merge(orderItem);
        em.getTransaction().commit();

        em.close();
    }

    private static void loadEntities(EntityManagerFactory emf) {
        System.out.println("-- Loading Customer --");
        EntityManager em = emf.createEntityManager();
        List<Customer> customers = em.createQuery("Select t from Customer t")
                                       .getResultList();
        customers.forEach(System.out::println);

        System.out.println("-- Loading OrderItem --");
        List<OrderItem> orderItems = em.createQuery("Select t from OrderItem t")
                                        .getResultList();
        orderItems.forEach(System.out::println);
        em.close();
    }

    public static void nativeQuery(EntityManager em, String s) {
        System.out.printf("---------------------------%n'%s'%n", s);
        Query query = em.createNativeQuery(s);
        List list = query.getResultList();
        for (Object o : list) {
            if (o instanceof Object[]) {
                System.out.println(Arrays.toString((Object[]) o));
            } else {
                System.out.println(o);
            }
        }
    }
}